bl_info = { 
    "name": "Show Collection/Object/Data Users of the Selected Object in the Viewport",
    "author": "1P2D",
    "version": (1, 1, 1),
    "blender": (4, 2, 0),
    "location": "View3D > Object > Viewport Display",
    "description": "Display the number of collection, object, and data users in the 3D viewport",
    "category": "Object Properties > Viewport Display > Show",
}

import bpy
import blf
from bpy_extras.view3d_utils import location_3d_to_region_2d
from bpy.types import Collection

handle = None

def register():
    global handle
    # Data Properties
    bpy.types.Object.show_data_name = bpy.props.BoolProperty(
        name="Data Name",
        description="Display the name of the data block",
        default=False
    )
    bpy.types.Object.show_data_users = bpy.props.BoolProperty(
        name="Data Users (Objects)",
        description="Display the number of objects sharing the data",
        default=True
    )
    bpy.types.Object.show_data_user_names = bpy.props.BoolProperty(
        name="Data User Names",
        description="Display the names of the objects sharing the data",
        default=False
    )
    
    # Object Users Properties
    bpy.types.Object.show_object_users = bpy.props.BoolProperty(
        name="Object Users",
        description="Display the number of users of the object",
        default=True
    )
    bpy.types.Object.show_object_user_names = bpy.props.BoolProperty(
        name="Object User Names",
        description="Display the scenes, collections and warnings related to object usage",
        default=False
    )

    bpy.types.OBJECT_PT_display.append(draw_users_properties)
    if handle is None:
        handle = bpy.types.SpaceView3D.draw_handler_add(
            draw_callback_px, (), 'WINDOW', 'POST_PIXEL')

def unregister():
    global handle
    # Remove properties
    del bpy.types.Object.show_data_name
    del bpy.types.Object.show_data_users
    del bpy.types.Object.show_data_user_names
    del bpy.types.Object.show_object_users
    del bpy.types.Object.show_object_user_names
    # Remove UI
    bpy.types.OBJECT_PT_display.remove(draw_users_properties)
    # Remove draw handler
    if handle is not None:
        bpy.types.SpaceView3D.draw_handler_remove(handle, 'WINDOW')
        handle = None

def draw_users_properties(self, context):
    layout = self.layout
    obj = context.object
    if obj is None:
        return
    # Data Properties
    layout.prop(obj, "show_data_name", text="Data Name")
    layout.prop(obj, "show_data_users", text="Data Users (Objects)")
    layout.prop(obj, "show_data_user_names", text="Data User Names")
    layout.separator()
    # Object Users
    layout.prop(obj, "show_object_users", text="Object Users")
    layout.prop(obj, "show_object_user_names", text="Object User Names")

def get_data_user_names(obj):
    """Retrieve a list of object names sharing the same data."""
    # For objects with data (like meshes, curves, etc)
    if obj.data is not None:
        return [obj.name] + [other_obj.name for other_obj in bpy.data.objects 
                if other_obj.data == obj.data and other_obj != obj]
    # For empties, instances, and linked assets (objects without data)
    return [obj.name]

def get_scene_references(obj):
    """Get list of scenes that reference the object via their Scene Collection."""
    return [scene.name for scene in bpy.data.scenes 
            if scene.collection.objects.get(obj.name)]

def get_collection_references(obj):
    """Get list of collections (excluding Scene Collections) that reference the object."""
    collections = []
    for col in obj.users_collection:
        if (col.objects.get(obj.name) 
            and not isinstance(col, bpy.types.LayerCollection)
            and col != bpy.context.scene.collection
            and not any(scene.collection == col for scene in bpy.data.scenes)):  # Exclude all Scene Collections
            warning = " (⚠)" if col.users > 1 else ""
            collections.append(f"{col.name}{warning}")
    return sorted(collections)

def get_multi_user_collections(obj):
    """Get all multi-user collections that contain this object."""
    multi_user_cols = []
    for col in obj.users_collection:
        if (col.objects.get(obj.name) and col.users > 1 
            and not isinstance(col, bpy.types.LayerCollection)
            and col != bpy.context.scene.collection
            and not any(scene.collection == col for scene in bpy.data.scenes)):
            multi_user_cols.append((col, col.users))
    return multi_user_cols

def has_temporary_references(obj):
    """Check if object has temporary references by comparing user count with actual references."""
    reference_count = len(get_scene_references(obj)) + len(get_collection_references(obj))
    if obj.library:
        reference_count += 1
    return obj.users > reference_count

def draw_callback_px():
    context = bpy.context
    if not isinstance(context.space_data, bpy.types.SpaceView3D) or not context.space_data.overlay.show_overlays:
        return

    font_id = 0
    blf.size(font_id, 11)
    theme = context.preferences.themes[0]
    view_3d_theme = theme.view_3d

    wire_color = (*view_3d_theme.wire, 1.0)
    object_active_color = (*view_3d_theme.object_active, 1.0)
    object_selected_color = (*view_3d_theme.object_selected, 1.0)

    x_offset = 10
    line_spacing = 3
    category_spacing = 15  # Additional spacing between categories

    # Find the window region
    window_region = None
    for area in context.screen.areas:
        if area.type == 'VIEW_3D':
            for region in area.regions:
                if region.type == 'WINDOW':
                    window_region = region
                    break
            if window_region:
                break
    if not window_region:
        return

    rv3d = area.spaces.active.region_3d

    for obj in context.visible_objects:
        try:
            if (obj.show_data_users or obj.show_data_user_names or obj.show_data_name 
                or obj.show_object_users or obj.show_object_user_names):
                
                obj_location = obj.matrix_world.translation
                coord_2d = location_3d_to_region_2d(window_region, rv3d, obj_location)

                if coord_2d:
                    color = (object_active_color if obj == context.view_layer.objects.active 
                             and obj.select_get() else (object_selected_color if obj.select_get() else wire_color))
                    blf.color(font_id, *color)
                    y_offset = coord_2d.y - 20  # Initial y-offset

                    # Data Group
                    if obj.show_data_name or obj.show_data_users or obj.show_data_user_names:
                        if obj.show_data_name:
                            data_name_text = f"Data Name: {obj.data.name}" if obj.data else "Data Name: N/A"
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, data_name_text)
                            y_offset -= (blf.dimensions(font_id, data_name_text)[1] + line_spacing)

                        if obj.show_data_users:
                            users_count = obj.data.users if obj.data else 0
                            user_text = "Data User" if users_count == 1 else "Data Users"
                            data_users_text = f"{user_text}: {users_count}" if obj.data else "Data User: N/A"
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, data_users_text)
                            y_offset -= (blf.dimensions(font_id, data_users_text)[1] + line_spacing)

                        if obj.show_data_user_names:
                            data_user_objects = get_data_user_names(obj)
                            name_text = "Object" if len(data_user_objects) == 1 else "Objects"
                            data_user_names_text = f"  • {name_text}: {', '.join(data_user_objects)}" if data_user_objects else "  • Object: None"
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, data_user_names_text)
                            y_offset -= (blf.dimensions(font_id, data_user_names_text)[1] + line_spacing)
                            
                    # Add spacing between Data and Object sections if any data property is shown
                    if obj.show_data_name or obj.show_data_users or obj.show_data_user_names:
                        y_offset -= category_spacing

                    # Object Users section
                    if obj.show_object_users:
                        scene_refs = get_scene_references(obj)
                        col_refs = get_collection_references(obj)
                        scene_count = len(scene_refs)
                        col_count = len(col_refs)
                        
                        # Handle singular/plural for scenes and collections
                        scene_text = f"{scene_count} scene" if scene_count == 1 else f"{scene_count} scenes"
                        col_text = f"{col_count} collection" if col_count == 1 else f"{col_count} collections"
                        
                        # Check for temporary references
                        has_temp = has_temporary_references(obj)
                        temp_indicator = "*" if has_temp else ""
                        
                        object_users_text = f"Object Users: {obj.users} ({scene_text} + {col_text}){temp_indicator}"
                        blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                        blf.draw(font_id, object_users_text)
                        y_offset -= (blf.dimensions(font_id, object_users_text)[1] + line_spacing)

                    # Object User Names section (detailed breakdown)
                    if obj.show_object_user_names:
                        scene_refs = get_scene_references(obj)
                        col_refs = get_collection_references(obj)
                        
                        # Scene References (show "none" if 0)
                        scene_text = f"  • Scenes: {', '.join(scene_refs)}" if scene_refs else "  • Scenes: none"
                        blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                        blf.draw(font_id, scene_text)
                        y_offset -= (blf.dimensions(font_id, scene_text)[1] + line_spacing)

                        # Collection References (show "none" if 0)
                        col_text = f"  • Collections: {', '.join(col_refs)}" if col_refs else "  • Collections: none"
                        blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                        blf.draw(font_id, col_text)
                        y_offset -= (blf.dimensions(font_id, col_text)[1] + line_spacing)

                        # Multi-User Collection Warning
                        multi_user_cols = get_multi_user_collections(obj)
                        if multi_user_cols:
                            col_label = "Collection" if len(multi_user_cols) == 1 else "Collections"
                            warning_text = f"⚠ Multi-User {col_label} Detected: Handle with care"
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, warning_text)
                            y_offset -= (blf.dimensions(font_id, warning_text)[1] + line_spacing)

                            for col, users in multi_user_cols:
                                col_text = f"  • {col.name} ({users} {'user' if users == 1 else 'users'})"
                                blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                                blf.draw(font_id, col_text)
                                y_offset -= (blf.dimensions(font_id, col_text)[1] + line_spacing)

        except Exception as e:
            print(f"Error processing object '{obj.name}': {e}")

if __name__ == "__main__":
    register()